/**
 * Created by xiaoqiang on 10/11/16.
 */
var Target;
(function (Target) {
  Target[Target['ios'] = 0] = 'ios';
  Target[Target['android'] = 1] = 'android';
  Target[Target['web'] = 2] = 'web';
}(Target || (Target = {})));
var Format;
(function (Format) {
  Format[Format['PNG'] = 0] = 'PNG';
  Format[Format['JPG'] = 1] = 'JPG';
  Format[Format['PDF'] = 2] = 'PDF';
  Format[Format['SVG'] = 3] = 'SVG';
  Format[Format['TIFF'] = 4] = 'TIFF';
  Format[Format['EPS'] = 5] = 'EPS';
}(Format || (Format = {})));
var App = function () {
    function App(_context) {
      this.document = _context.document;
      this.context = _context;  //this.sketch = this.context.api();
    }
    App.prototype.export = function (options) {
      if (options == null) {
        return;
      }
      this.selection = this.document.selectedLayers();
      if (this.selection.layers) {
        // adapter for version 42
        this.selection = this.selection.layers();
      }
      log('cutterman export: ' + options);
      var optionObj = JSON.parse(options);
      this.configure = optionObj['configure'];
      this.target = optionObj['target'];
      if (this.configure['output'] == '') {
        this.document.showMessage('\u8bf7\u5148\u9009\u62e9\u5bfc\u51fa\u76ee\u6807\u4f4d\u7f6e');
        return;
      }
      if (this.selection.count() == 0) {
        this.document.showMessage('\u8bf7\u5148\u9009\u62e9\u9700\u8981\u5bfc\u51fa\u7684\u56fe\u5c42');
        return;
      }
      this.retriveCanvasSize();
      for (var i = 0; i < this.selection.count(); i++) {
        var layer = this.selection[i];
        this.initSliceArtboard();
        this.proccessLayer2(layer);
        this.removeSliceArtboard();
      }
      this.document.showMessage('\u5bfc\u51fa\u6210\u529f');
    };
    App.prototype.proccessLayer2 = function (layer) {
      var newLayer = layer.duplicate();
      this.artboard.addLayers([newLayer]);
      layer.parentGroup().removeLayer(newLayer);
      var frame = this.getFrame(newLayer);
      log('layer frame: (' + frame.x() + ', ' + frame.y() + ', ' + frame.width() + ', ' + frame.height() + ')');
      newLayer.exportOptions().removeAllExportFormats();
      var name = newLayer.name();
      name = name.toLowerCase().replace(/\s/, '_').replace(/-+/g, '_');
      var fixWidth = this.configure['width'] == '' ? 0 : parseInt(this.configure['width']);
      var fixHeight = this.configure['height'] == '' ? 0 : parseInt(this.configure['height']);
      var ratios = this.calculateRatio();
      var suffixs = this.getTargetSuffix();
      var dpis = this.getDpiList();
      for (var i = 0; i < dpis.length; i++) {
        var dpi = dpis[i];
        if (dpi == 1) {
          var ratio = ratios[i];
          var suffix = suffixs[i].toLocaleLowerCase();
          var format = Format[this.configure['format']].toLocaleLowerCase();
          var fileName = this.getOutputPath(name, suffixs[i].toLocaleLowerCase());
          log('Processing ' + name + ' ' + ratio + ' to ' + fileName);
          var sliceLayer = MSSliceLayer.new();
          sliceLayer.setName(name + '_slice');
          var sliceFrame = sliceLayer.frame();
          var x = Math.round(frame.x());
          var y = Math.round(frame.y());
          var width = Math.round(frame.width());
          var height = Math.round(frame.height());
          if (fixWidth > width && fixHeight > height) {
            x = frame.x() + width / 2 - fixWidth / 2;
            y = frame.y() + height / 2 - fixHeight / 2;
            width = fixWidth;
            height = fixHeight;
          }
          sliceFrame.setX(Math.round(x));
          sliceFrame.setY(Math.round(y));
          sliceFrame.setWidth(Math.round(width));
          sliceFrame.setHeight(Math.round(height));
          this.artboard.addLayer(sliceLayer);
          log('set slice rect: (' + x + ', ' + y + ', ' + width + ', ' + width + ')');
          var size = sliceLayer.exportOptions().addExportFormat();
          size.setName(suffix);
          size.setScale(ratio);
          size.fileFormat = format;
          sliceLayer.exportOptions().addExportFormat();
          sliceLayer.exportOptions().removeExportFormat(sliceLayer.exportOptions().exportFormats().objectAtIndex(sliceLayer.exportOptions().exportFormats().count() - 1));
          var slice = MSExportRequest.exportRequestsFromExportableLayer(sliceLayer).firstObject();
          slice.scale = ratio;
          slice.setShouldTrim(0);
          slice.format = format;
          this.document.saveArtboardOrSlice_toFile(slice, fileName);
        }
      }
    };
    App.prototype.retriveCanvasSize = function () {
      var artboard = this.document.currentPage().currentArtboard();
      var frame = artboard.frame();
      this.canvas = [
        frame.width(),
        frame.height()
      ];
    };
    App.prototype.initSliceArtboard = function () {
      this.artboard = MSArtboardGroup.new();
      var page = this.document.currentPage();
      var bounds = page.contentBounds();
      var right = bounds.origin.x + bounds.size.width;
      var bottom = bounds.origin.y + bounds.size.height;
      var frame = this.artboard.frame();
      frame.setX(right + 20);
      frame.setY(bottom + 20);
      frame.setWidth(this.canvas[0]);
      frame.setHeight(this.canvas[1]);
      this.document.currentPage().addLayers([this.artboard]);
    };
    App.prototype.removeSliceArtboard = function () {
      if (this.artboard != null) {
        this.document.currentPage().removeLayer(this.artboard);
        this.artboard = null;
      }
    };
    App.prototype.getDpiList = function () {
      var format = parseInt(this.configure['format']);
      if (format == Format.PNG) {
        return this.configure['dpi'];
      }
      return [1];
    };
    App.prototype.getTargetSuffix = function () {
      if (parseInt(this.configure['format']) != Format.PNG) {
        return [''];
      }
      if (this.target == Target.ios) {
        return [
          '@3x',
          '@2x',
          ''
        ];
      } else if (this.target == Target.android) {
        return [
          'XXXHDPI',
          'XXHDPI',
          'XHDPI',
          'HDPI',
          'MDPI'
        ];
      }
      return [''];
    };
    App.prototype.getOutputPath = function (name, suffix) {
      var format = parseInt(this.configure['format']);
      var formatStr = Format[format].toLocaleLowerCase();
      if (this.target == Target.ios) {
        return this.configure['output'] + '/' + name + suffix + '.' + formatStr;
      } else if (this.target == Target.android) {
        if (suffix != '') {
          suffix = '-' + suffix;
        }
        return this.configure['output'] + '/drawable' + suffix + '/' + name + '.' + formatStr;
      } else {
        return this.configure['output'] + '/' + name + '.' + formatStr;
      }
    };
    App.prototype.calculateRatio = function () {
      if (parseInt(this.configure['format']) != Format.PNG) {
        return [1];
      }
      var width = Math.min(this.canvas[0], this.canvas[1]);
      if (this.target == Target.ios) {
        switch (width) {
        case 375:
          return [
            3,
            2,
            1
          ];
        case 720:
          var ratio = 750 / 720;
          return [
            3 * ratio / 2,
            1 * ratio,
            1 * ratio / 2
          ];
        case 1080:
        // 3X
        case 1125:
        case 1242:
          return [
            1,
            2 / 3,
            1 / 3
          ];
        default:
          return [
            3 / 2,
            1,
            1 / 2
          ];
        }
      } else if (this.target == Target.android) {
        // [XXXHDPI, XXHDPI, XHDPI, HDPI, MDPI]
        switch (width) {
        case 375:
          var ratio1 = 320 / 375;
          return [
            4 * ratio1,
            3 * ratio1,
            2 * ratio1,
            3 * ratio1 / 2,
            1 * ratio1
          ];
        case 750:
          var ratio = 720 / 750;
          return [
            2 * ratio,
            1.5 * ratio,
            1 * ratio,
            0.75 * ratio,
            0.5 * ratio
          ];
        case 720:
          return [
            2,
            1.5,
            1,
            0.75,
            0.5
          ];
        case 1080:
          return [
            4 / 3,
            1,
            2 / 3,
            1 / 2,
            1 / 3
          ];
        case 1440:
          return [
            1,
            3 / 4,
            1 / 2,
            3 / 8,
            1 / 4
          ];
        case 480:
          return [
            8 / 3,
            2,
            4 / 3,
            1,
            2 / 3
          ];
        case 320:
          return [
            4,
            3,
            2,
            3 / 2,
            1
          ];
        default:
          return [
            2,
            1.5,
            1,
            0.75,
            0.5
          ];
        }
      }
      return [1];
    };
    App.prototype.getFrame = function (layer) {
      var artboard = this.artboard;
      //this.document.currentPage().currentArtboard();
      var artboardFrame = artboard.absoluteRect();
      var frame = layer.absoluteRect();
      var x = frame.x() - artboardFrame.x();
      var y = frame.y() - artboardFrame.y();
      return MSRect.alloc().initWithRect(NSMakeRect(x, y, frame.width(), frame.height()));
    };
    return App;
  }();